import React, { useEffect, useMemo } from 'react';
import {
  ErrorOption,
  FieldArrayWithId,
  UseFieldArrayReturn,
  useWatch,
} from 'react-hook-form';
import { FieldBase } from '@digigov/form/Field/FieldBase';
import { CalculatedField } from '@digigov/form/Field/types';
import {
  FIELD_COMPONENTS,
  ALTERNATIVE_COMPONENTS,
} from '@digigov/form/Field/utils';
import { calculateField } from '@digigov/form/Field/utils/calculateField';
import { useField } from '@digigov/form/Field/utils/useField';
import { FieldObject } from '@digigov/form/FieldObject';
import { Fieldset } from '@digigov/form/Fieldset';
import type {
  MultiplicityProps,
  StashedObject,
} from '@digigov/form/MultiplicityField/types';
import { FieldSpec } from '@digigov/form/types';
import { Button } from '@digigov/ui/form/Button';
import { RadioConditional } from '@digigov/ui/form/RadioContainer';

export interface AddObjectsProps
  extends Omit<MultiplicityProps, 'key' | 'component'> {
  fields: FieldArrayWithId[];
  error?: ErrorOption;
  append: UseFieldArrayReturn['append'];
  remove: UseFieldArrayReturn['remove'];
  stashedObjects: StashedObject[];
  onStash: (arg0: StashedObject) => void;
  children?: React.ReactNode;
}

export const AddObjects: React.FC<AddObjectsProps> = ({
  name,
  fields,
  formState,
  extra,
  append,
  remove,
  trigger,
  clearErrors,
  children,
  setValue,
  stashedObjects,
  getValues,
  onStash,
  error,
  Field,
}) => {
  const currentIndex = fields.length > 0 ? fields.length - 1 : fields.length;
  const currentName = `${name}.${currentIndex}`;
  const radioField: FieldSpec = {
    key: 'needs-more',
    type: 'choice:single',
    label: {
      primary: extra?.label.question.title,
    },
    required: true,
    validators: [
      {
        name: 'is-locked',
        message: 'form.error.needs-more',
        test: (data) => {
          return data === 'no';
        },
      },
    ],
    extra: {
      options: [
        {
          label: { primary: extra?.label.question.yes },
          id: `radio-input-${name}-yes`,
          value: 'yes',
          selected: function RadioSelected() {
            return (
              <RadioConditional
                hidden={
                  needsMore !== 'yes' ||
                  (extra?.max && extra?.max - stashedObjects.length === 0)
                }
              >
                {needsMore === 'yes' && fields.length > 0 && (
                  <>
                    <FieldObject
                      name={currentName}
                      error={error && error[currentIndex]}
                      register={register}
                      formState={formState}
                      control={control}
                      {...extra?.of}
                      label={
                        extra?.label.question.objectLabel || extra?.of.label
                      }
                    />
                    <Button
                      color="secondary"
                      name={`${currentName}-add-object`}
                      onClick={async (e) => {
                        e.preventDefault();
                        const nestedFields = extra?.of.extra.fields.map(
                          (f) => `${currentName}.${f.key}`
                        );
                        const result = await trigger(nestedFields);
                        if (result) {
                          setValue('needs-more', '');
                          onStash(getValues(currentName));
                        }
                      }}
                    >
                      {extra?.label.object.add}
                    </Button>
                  </>
                )}
              </RadioConditional>
            );
          },
        },
        {
          label: { primary: extra?.label.question.no },
          id: `radio-input-${name}-no`,
          value: 'no',
        },
      ],
    },
  };

  const {
    field,
    control,
    register,
    reset,
    error: needsMoreError,
  } = useField(radioField.key, radioField?.type ? radioField : null);

  const calculatedField: CalculatedField = useMemo(
    () =>
      calculateField(children, field, FIELD_COMPONENTS, ALTERNATIVE_COMPONENTS),
    [field]
  );
  const needsMore = useWatch({ name: field.key, control });

  useEffect(() => {
    if (needsMore === 'yes') {
      append({ afm: '', firstName: '', lastName: '' });
    }
    if (needsMore === 'no' && stashedObjects.length < fields.length) {
      remove(currentIndex);
    }
    if (needsMore === '') clearErrors('needs-more');
  }, [needsMore]);

  // TODO: Refactor types so that label is always required, the following
  // assignment is a temp fix for typescript build to run
  calculatedField.label = calculatedField.label || {};
  if (extra?.length) {
    calculatedField.label.secondary = `Έχετε προσθέσει ${
      stashedObjects.length
    } εγγραφές και σας απομένουν ακόμα ${
      extra.length - stashedObjects.length
    } εγγραφές.`;
  } else if (extra?.min && extra?.max) {
    calculatedField.label.secondary = `Το πεδίο πρέπει να έχει ακόμα ${
      extra?.min - stashedObjects.length
    } και το μέγιστο ${
      extra?.max
    } εγγραφές. Προσθέστε μια ακόμα επιλέγοντας την απάντηση «Ναι» στην επόμενη ερώτηση.`;
  } else if (extra?.min) {
    calculatedField.label.secondary = `Το πεδίο πρέπει να έχει ακόμα ${
      extra?.min - stashedObjects.length
    } εγγραφές. Προσθέστε μια ακόμα επιλέγοντας την απάντηση «Ναι» στην επόμενη ερώτηση.`;
  } else if (extra?.max) {
    calculatedField.label.secondary = ` Έχετε προσθέσει ${
      stashedObjects.length
    } εγγραφές και σας απομένουν ${
      extra.max - stashedObjects.length
    } επιπλέον εγγραφές.`;
  }
  return (
    <Fieldset>
      <FieldBase
        {...calculatedField}
        name={calculatedField.key}
        control={control}
        register={register}
        reset={reset}
        Field={Field}
        error={
          extra?.max - stashedObjects.length === 0 ||
          extra?.length - stashedObjects.length === 0
            ? error
            : needsMoreError
        }
      />
    </Fieldset>
  );
};

export default AddObjects;
