import React, { useMemo, useState } from 'react';
import dayjs from 'dayjs';
import customParseFormat from 'dayjs/plugin/customParseFormat';
import { UncontrolledFieldProps } from '@digigov/form/Field/types';
import {
  DateInputContainer,
  DateInputItem,
} from '@digigov/ui/form/DateInputContainer';
import { useTranslation } from '@digigov/ui/i18n';
dayjs.extend(customParseFormat);

export interface DateInputProps extends Omit<UncontrolledFieldProps, 'extra'> {
  extra?: {
    max?: string;
    min?: string;
  };
}

function useDate(value, onChange) {
  const [year, month, day] = useMemo(() => {
    if (!value || value.length === 0) {
      onChange('');
      return ['', '', ''];
    }
    const [day, month, year] = value.split('/');
    return [year, month, day];
  }, [value]);

  function set(year, month, day) {
    const value = `${day || ''}/${month || ''}/${year || ''}`;
    if (!year && !month && !day) {
      onChange(null);
    } else {
      onChange(value);
    }
  }

  function setYear(evt) {
    const val = evt.target.value.trim();
    set(val, month, day);
  }

  function setMonth(evt, cast) {
    let val = evt.target.value.trim();
    if (cast && val && val.length === 1) {
      val = '0' + val;
    }
    set(year, val, day);
  }

  function setDay(evt, cast) {
    let val = evt.target.value.trim();
    if (cast && val && val.length === 1) {
      val = '0' + val;
    }
    set(year, month, val);
  }

  return {
    day,
    month,
    year,
    setYear,
    setMonth,
    setDay,
  };
}

const VALID_FORMATS = ['DD/MM/YYYY'];

const makeDate = (val) => {
  const date = dayjs(val, VALID_FORMATS, true);
  if (date.isValid()) {
    return date.format('DD/MM/YYYY');
  }
  return '';
};

const DatePart = ({ label, ...props }) => {
  return <DateInputItem {...props}>{label}</DateInputItem>;
};

export const DateInput = ({ name, ...props }) => {
  const { t } = useTranslation();
  const [initial] = useState(props.value);
  const value = useMemo(() => {
    if (initial === props.value) {
      return makeDate(props.value);
    }
    return props.value;
  }, [props.value, initial]);

  const date = useDate(value || makeDate(props.defaultValue), props.onChange);

  return (
    <DateInputContainer>
      <DatePart
        label={t('form.label.day')}
        onChange={date.setDay}
        onBlur={(e) => date.setDay(e, true)}
        value={date.day}
        width={2}
        name={`${name}-day`}
        maxLength="2"
        disabled={props.disabled}
        aria-required={props['aria-required']}
        error={!date.day ? props.error : false}
      />
      <DatePart
        label={t('form.label.month')}
        onChange={date.setMonth}
        onBlur={(e) => date.setMonth(e, true)}
        value={date.month}
        width={2}
        name={`${name}-month`}
        maxLength="2"
        disabled={props.disabled}
        aria-required={props['aria-required']}
        error={!date.month ? props.error : false}
      />
      <DatePart
        label={t('form.label.year')}
        onChange={date.setYear}
        value={date.year}
        width={4}
        name={`${name}-year`}
        maxLength="4"
        disabled={props.disabled}
        aria-required={props['aria-required']}
        error={!date.year ? props.error : false}
      />
    </DateInputContainer>
  );
};

export default DateInput;
