import React, { useState, useCallback } from 'react';
import { UncontrolledFieldProps } from '@digigov/form/Field/types';
import { Button, ButtonGroup } from '@digigov/ui/form/Button';
import { FileUpload, FileUploadContainer } from '@digigov/ui/form/FileUpload';
import { useTranslation } from '@digigov/ui/i18n';
import { Hint } from '@digigov/ui/typography/Hint';
import { Paragraph } from '@digigov/ui/typography/Paragraph';
import { Base } from '@digigov/ui/utils/Base';
export interface ImageProps {
  src?: any;
}

export interface Limit {
  maxSize?: number;
  maxWidth?: number;
  maxHeight?: number;
  minWidth?: number;
  minHeight?: number;
}

export const Image: React.FC<ImageProps> = React.memo(function Image({ src }) {
  return <Base as="img" src={src} className="ds-image--ratio" />;
});
export interface ImageInputProps extends Omit<UncontrolledFieldProps, 'extra'> {
  extra?: {
    className?: string;
    multiple?: boolean;
    accept?: string;
    limit?: Limit;
  };
  reset: (values?: any) => void;
  onChange: (e: React.ChangeEvent<HTMLInputElement>) => void;
}

export const ImageInput: React.FC<ImageInputProps> = React.forwardRef(
  function ImageInput({ name, disabled, onChange, reset, ...props }, ref: any) {
    const { t } = useTranslation();
    const [selectedImage, setSelectedImage] = useState<File | null>(null);
    const [imageUrl, setImageUrl] = useState<string | null>(null);
    const handleImageChange = useCallback(
      (e: React.ChangeEvent<HTMLInputElement>) => {
        e.preventDefault();
        const target = e.target as HTMLInputElement;
        const selectedFile = target.files?.[0];

        if (selectedFile) {
          setSelectedImage(selectedFile);
          setImageUrl(URL.createObjectURL(selectedFile));
        }
        onChange(e);
      },
      [selectedImage]
    );

    const handleDeleteImage = useCallback(
      (event) => {
        event.preventDefault();
        reset({ [name]: '' });
        setSelectedImage(null);
        setImageUrl('');
      },
      [selectedImage]
    );

    return (
      <>
        <FileUploadContainer>
          {selectedImage !== null ? (
            <>
              <Paragraph>
                <Base as="b">{t('upload.image')}:</Base>{' '}
                {`${selectedImage?.name}`}
              </Paragraph>
            </>
          ) : (
            <Hint className="ds-dashed--border">{t('upload.no_image')}</Hint>
          )}
          <>
            {selectedImage !== null && <Image src={imageUrl} />}
            <ButtonGroup marginTop={2}>
              {selectedImage !== null && (
                <Button variant="link" onClick={handleDeleteImage}>
                  {t('upload.reset_file')}
                </Button>
              )}
              <FileUpload
                ref={ref}
                hasFiles={selectedImage !== null}
                name={name}
                disabled={disabled}
                accept="image/*"
                onChange={handleImageChange}
                {...{ ...props, required: undefined, type: 'file' }}
              >
                {t('upload.choose_image')}
              </FileUpload>
            </ButtonGroup>
          </>
        </FileUploadContainer>
      </>
    );
  }
);

export default ImageInput;
