import React from 'react';
import { useWatch } from 'react-hook-form';
import { UncontrolledFieldProps } from '@digigov/form/Field/types';
import { TextArea } from '@digigov/ui/form/TextArea';
import { TextInput } from '@digigov/ui/form/TextInput';
import { useTranslation } from '@digigov/ui/i18n';
import { Hint } from '@digigov/ui/typography/Hint';

const TYPES_MAP = {
  string: 'text',
  int: 'text',
  text: 'text',
  date: 'date',
};

export interface InputProps extends Omit<UncontrolledFieldProps, 'extra'> {
  extra?: {
    fullWidth?: boolean;
    multiline?: boolean;
    className?: string;
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    rows?: any;
    limit: {
      min?: number;
      max?: number;
    };
  };
}

export const Input: React.ExoticComponent<InputProps> = React.forwardRef(
  function WrappedInput({ name, control, type, extra, ...props }, ref: never) {
    // eslint-disable-next-line prefer-const
    let { multiline = false, rows, limit } = extra || {};
    const { className } = extra || {};
    // if enforced to multiline use true. Derive from type otherwise.
    multiline = multiline === true || type === 'text';
    // use explicit rows value if set. Derive from multiline otherwise.
    rows = rows || (multiline ? 4 : 1);
    // translate dilosi type to HTML Input type
    const fieldType = TYPES_MAP[type || 'text'] || 'text';
    const { t } = useTranslation();
    const currentValue: string | '' = useWatch({ control, name });

    const getRemainingChars = (currentValue, limit) => {
      if (currentValue) {
        return limit?.max - currentValue.length;
      }
      return limit?.max;
    };

    const constructRemainingText = (currentValue, limit) => {
      const remainingChars = getRemainingChars(currentValue, limit);
      let remainingText =
        Math.abs(remainingChars) === 1
          ? t('form.info.text.character')
          : t('form.info.text.characters');
      remainingText +=
        remainingChars > 0
          ? ` ${t('form.info.text.remaining')}.`
          : ` ${t('form.info.text.too_many')}.`;
      return remainingText;
    };

    if (multiline === true) {
      return (
        <>
          <TextArea
            name={name}
            className={className}
            rows={rows}
            ref={ref}
            {...{
              ...props,
              reset: undefined,
              required: undefined,
              type: fieldType,
            }}
          />
          {limit?.max && (
            <Hint display={'flex'}>
              {t('form.info.text.you_have')}
              <Hint fontWeight="bold">
                &nbsp;{`${Math.abs(getRemainingChars(currentValue, limit))}`}
                &nbsp;
              </Hint>
              {constructRemainingText(currentValue, limit)}
            </Hint>
          )}
        </>
      );
    } else {
      return (
        <TextInput
          name={name}
          id={name}
          type={fieldType}
          data-type={type}
          className={className}
          ref={ref}
          {...{ ...props, reset: undefined, required: undefined }}
        />
      );
    }
  }
);

export default Input;
