import React, { useState, useCallback } from 'react';
import SingleCharacterInputs, {
  SingleCharacterInput,
} from '@digigov/ui/form/SingleCharacterInputs';

export const RE_DIGIT = new RegExp(/^\d+$/);
export const REMOVE_SPACES = new RegExp(/\s+/g);

export function useOtp(onChange, maxLength) {
  const [otpValues, setOtpValues] = useState(Array<string>(maxLength).fill(''));
  const handleChange = useCallback(
    (otp: string[]) => {
      setOtpValues(otp);
      const otpValue = otp.join('');
      onChange(otpValue);
    },
    [onChange]
  );

  const setOtp = (item: string, idx) => {
    const updatedOTPValues = [...otpValues];
    if (item.length === 1 || item.length === 0) {
      updatedOTPValues[idx] = item[0] || '';
      handleChange(updatedOTPValues);
    } else if (item.length === maxLength) {
      const valueArray = item.split('').slice(0, maxLength);
      handleChange(valueArray);
    }
  };

  const setSelectionRange = useCallback((target) => {
    return target.setSelectionRange(0, target.value.length);
  }, []);

  const focusToNextInput = useCallback((target: HTMLElement) => {
    const nextElement = target.nextElementSibling as HTMLInputElement | null;
    if (nextElement) {
      nextElement.focus();
    }
  }, []);

  const focusToPrevInput = useCallback((target: HTMLElement) => {
    const previousElement =
      target.previousElementSibling as HTMLInputElement | null;
    if (previousElement) {
      previousElement.focus();
    }
  }, []);

  const handleKeyDown = useCallback(
    (e: React.KeyboardEvent<HTMLInputElement>, idx: number) => {
      const { key } = e;
      const target = e.target as HTMLInputElement;
      let targetValue = target.value;
      setSelectionRange(target);
      if (key === 'ArrowRight' || key === 'ArrowDown') {
        e.preventDefault();
        return focusToNextInput(target);
      }
      if (key === 'ArrowLeft' || key === 'ArrowUp') {
        e.preventDefault();
        return focusToPrevInput(target);
      }
      if (e.key === 'Backspace' || e.key === 'Delete') {
        if (targetValue !== '') {
          targetValue = '';
        } else {
          focusToPrevInput(target);
        }
        setOtp(targetValue, idx);
        return e.preventDefault();
      }
    },
    [setOtp, focusToPrevInput, focusToNextInput]
  );

  const handleFocus = useCallback(
    (e: React.FocusEvent<HTMLInputElement>) => {
      const { target } = e;
      setSelectionRange(target);
    },
    [setSelectionRange]
  );
  const handleOtpChange = useCallback(
    (e: React.ChangeEvent<HTMLInputElement>, idx: number) => {
      const target = e.target;
      const targetValue = target.value;
      const isTargetValueDigit = RE_DIGIT.test(targetValue);
      // We want to pass the empty string when the value is deleted
      // emptry string replaces the deleted value
      if (isTargetValueDigit || targetValue === '') {
        setOtp(targetValue, idx);
        if (target.value.length > 1) {
          target.blur();
        }
        focusToNextInput(target);
      }
    },
    [setOtp, handleKeyDown, handleFocus]
  );

  const handlePaste = useCallback(
    (e: React.ClipboardEvent<HTMLInputElement>) => {
      e.preventDefault();
      const pastedData = e.clipboardData
        .getData('text/plain')
        .replace(REMOVE_SPACES, '');
      if (pastedData) {
        setOtp(pastedData, 0);
      }
    },
    []
  );
  return {
    otpValues,
    handleOtpChange,
    handleKeyDown,
    handleFocus,
    handlePaste,
  };
}

const SingleCharacterInputPart = ({ ...props }) => {
  return <SingleCharacterInput {...props} />;
};

export const OtpInput = ({ name, maxLength = 6, ...props }) => {
  const otp = useOtp(props.onChange, maxLength);
  return (
    <SingleCharacterInputs>
      {otp.otpValues.map((digit, idx) => (
        <SingleCharacterInputPart
          name={`${name}-${idx}`}
          type="text"
          key={idx}
          autoComplete="one-time-code"
          pattern="\d{1}"
          maxLength={maxLength}
          value={digit}
          disabled={props.disabled}
          aria-required={props['aria-required']}
          onChange={(e) => otp.handleOtpChange(e, idx)}
          onInput={(e) => {
            if (e.target.value === digit) {
              otp.handleOtpChange(e, idx);
            }
          }}
          onKeyDown={(e) => otp.handleKeyDown(e, idx)}
          onFocus={otp.handleFocus}
          onPaste={otp.handlePaste}
          error={!digit ? props.error : false}
        />
      ))}
    </SingleCharacterInputs>
  );
};

export default OtpInput;
