import { MutableRefObject, useMemo } from 'react';
import * as yup from 'yup';
import { AnyObjectSchema, StringSchema } from 'yup';
import { RequiredArraySchema } from 'yup/lib/array';
import Lazy from 'yup/lib/Lazy';
import { MixedSchema } from 'yup/lib/mixed';
import { TypeOfShape, AssertsShape } from 'yup/lib/object';
import { AnyObject } from 'yup/lib/types';
import { FieldSpec } from '@digigov/form/types';
import { ValidatorSchema } from '@digigov/form/validators/types';
import {
  getYupObjectShape,
  AFM_VALIDATOR,
  FILE_MAX_SIZE_VALIDATOR,
  IBAN_VALIDATOR,
  OTP_VALIDATOR,
  MOBILE_PHONE_VALIDATOR,
  PHONE_NUMBER_VALIDATOR,
  TEXT_LIMIT_VALIDATOR,
  POSTALCODE_VALIDATOR,
  UUID4_VALIDATOR,
  IMAGE_DIMENSION_VALIDATOR,
  DATE_VALIDATOR,
  LATER_THAN_VALIDATOR,
  EARLIER_THAN_VALIDATOR,
} from '@digigov/form/validators/utils';
import { INT_VALIDATOR } from '@digigov/form/validators/utils/int';
import { NUMBER_VALIDATOR } from '@digigov/form/validators/utils/number';

export const getYUPTypeMap = (): Record<string, any> => {
  const yupTypeMap = {
    file: (field): MixedSchema => {
      return yup
        .mixed()
        .transform((value) => {
          if (!value.length) {
            return null;
          }
          return value;
        })
        .nullable()
        .test(FILE_MAX_SIZE_VALIDATOR(field));
    },
    string: (field): yup.StringSchema => {
      if (field.trim) {
        return yup.string().trim().required(field.required);
      } else {
        return yup.string();
      }
    },
    boolean: (): yup.BooleanSchema<
      boolean | null | undefined,
      AnyObject,
      boolean | null | undefined
    > => {
      return yup
        .boolean()
        .nullable()
        .transform((value) => {
          if (value === '' || typeof value === 'string') return null;
          if (typeof value === 'boolean') return value;
          if (value === 0 || value === 1) return Boolean(value);
          return value;
        });
    },
    object: (
      field
    ):
      | yup.ObjectSchema<any, AnyObject, TypeOfShape<any>, AssertsShape<any>>
      | Lazy<any, unknown> => {
      return getYupObjectShape(field.extra.fields, yupTypeMap);
    },
    array: (
      field
    ):
      | yup.ArraySchema<any, AnyObject, any[] | undefined>
      | RequiredArraySchema<any, AnyObject, any[] | undefined> => {
      if (typeof field.extra.of === 'object') {
        // else if the field is an object then it is a scalar type
        // eg. number, string etc.
        let arrayValidator = yup
          .array()
          .of(yupTypeMap[field.extra.of.type](field.extra.of))
          .required(field.required);
        if (field.extra.length)
          arrayValidator = arrayValidator.length(field.extra.length, {
            key: 'form.error.array.length',
            context: {
              length: field.extra.length,
            },
          });
        if (field.extra.min)
          arrayValidator = arrayValidator.min(field.extra.min, {
            key: 'form.error.array.min',
            context: {
              min: field.extra.min,
            },
          });
        if (field.extra.max)
          arrayValidator = arrayValidator.max(field.extra.max, {
            key: 'form.error.array.max',
            context: {
              max: field.extra.max,
            },
          });
        return arrayValidator;
      } else if (typeof field.extra.of === 'string') {
        return yup.array().of(yupTypeMap[field.extra.of]());
      } else {
        // in any other case it will be string
        return yup.array().of(yup.string());
      }
    },
    number: (): MixedSchema =>
      yup
        .mixed()
        .transform((_, val) => (val !== '' ? Number(val) : undefined))
        .test(NUMBER_VALIDATOR()),
    int: (): MixedSchema =>
      yup
        .mixed()
        .transform((_, val) => (val !== '' ? Number(val) : undefined))
        .test(NUMBER_VALIDATOR())
        .test(INT_VALIDATOR()),
    email: (field): yup.StringSchema =>
      field.trim
        ? yup.string().trim().email('form.error.email')
        : yup.string().email('form.error.email'),
    afm: (field): yup.StringSchema =>
      field.trim
        ? yup.string().trim().test(AFM_VALIDATOR)
        : yup.string().test(AFM_VALIDATOR),
    uuid4: (field): yup.StringSchema =>
      field.trim
        ? yup.string().trim().test(UUID4_VALIDATOR)
        : yup.string().test(UUID4_VALIDATOR),
    iban: (field): yup.StringSchema =>
      field.trim
        ? yup.string().trim().test(IBAN_VALIDATOR(field))
        : yup.string().trim().test(IBAN_VALIDATOR(field)),
    otp: (field): yup.StringSchema =>
      yup.string().trim().test(OTP_VALIDATOR(field)),
    text: (field): yup.StringSchema => {
      if (field?.extra?.limit?.max || !!field?.extra?.limit?.min) {
        return field.trim
          ? yup.string().trim().test(TEXT_LIMIT_VALIDATOR(field))
          : yup.string().test(TEXT_LIMIT_VALIDATOR(field));
      } else {
        return field.trim ? yup.string().trim() : yup.string();
      }
    },
    image: (field): MixedSchema => {
      return yup
        .mixed()
        .transform((value) => {
          if (!value.length || value.length === 0) {
            return null;
          }
          return value;
        })
        .nullable()
        .test(FILE_MAX_SIZE_VALIDATOR(field))
        .test(IMAGE_DIMENSION_VALIDATOR(field));
    },
    postal_code: (field): yup.StringSchema =>
      field.trim
        ? yup.string().trim().test(POSTALCODE_VALIDATOR(field))
        : yup.string().test(POSTALCODE_VALIDATOR(field)),
    mobile_phone: (field): yup.StringSchema =>
      field.trim
        ? yup.string().trim().test(MOBILE_PHONE_VALIDATOR)
        : yup.string().test(MOBILE_PHONE_VALIDATOR),
    phone_number: (field): yup.StringSchema =>
      field.trim
        ? yup.string().trim().test(PHONE_NUMBER_VALIDATOR(field))
        : yup.string().test(PHONE_NUMBER_VALIDATOR(field)),
    'choice:multiple': (): yup.ArraySchema<
      yup.StringSchema<string | undefined, AnyObject, string | undefined>,
      AnyObject,
      (string | undefined)[] | null | undefined,
      (string | undefined)[] | null | undefined
    > => yup.array().of(yup.string()).nullable(),
    'choice:single': (): StringSchema<
      string | null | undefined,
      AnyObject,
      string | null | undefined
    > => yup.string().nullable(),
    date: (
      field
    ): yup.StringSchema<
      string | null | undefined,
      AnyObject,
      string | null | undefined
    > => {
      let schema = yup.string().nullable();
      if (field?.extra?.min) {
        schema = schema.test(LATER_THAN_VALIDATOR(field));
      }
      if (field?.extra?.max) {
        schema = schema.test(EARLIER_THAN_VALIDATOR(field));
      } else {
        schema = schema.test(DATE_VALIDATOR);
      }
      return schema;
    },
  };
  return yupTypeMap;
};

export interface MutableRefObjectProps {}
// Create a yup validation schema from given fields input
export function useValidationSchema(
  fields: FieldSpec[] | MutableRefObject<MutableRefObjectProps>,
  validatorRegistry?: Record<string, ValidatorSchema[]>
): Lazy<any, unknown> | AnyObjectSchema | void {
  return useMemo(() => {
    const yupTypeMap = getYUPTypeMap();
    return getYupObjectShape(fields, yupTypeMap, validatorRegistry);
  }, []);
}
