import dayjs from 'dayjs';
import customParseFormat from 'dayjs/plugin/customParseFormat';
import { FieldSpec } from '@digigov/form/types';
import { ValidatorSchema } from '@digigov/form/validators/types';

dayjs.extend(customParseFormat);

const VALID_DATE_FORMATS = ['DD/MM/YYYY'];
const DATE_CACHE = {};

const getDate = (v: string) => {
  return DATE_CACHE[v] ? DATE_CACHE[v] : dayjs(v, VALID_DATE_FORMATS, true);
};

// Function to check partial date completion
const checkPartialDate = (value: string) => {
  console.log("value.split('/')", value.split('/'));
  const [day, month, year] = value.split('/');
  if (day && !month && !year) {
    return 'form.error.date.invalid_month_year';
  }
  if (!day && month && !year) {
    return 'form.error.date.invalid_day_year';
  }
  if (!day && !month && year) {
    return 'form.error.date.invalid_day_month';
  }
  if (day && month && !year) {
    return 'form.error.date.invalid_year';
  }
  if (day && !month && year) {
    return 'form.error.date.invalid_month';
  }
  if (!day && month && year) {
    return 'form.error.date.invalid_day';
  } else {
    return 'form.error.date.invalid';
  }
};

export const LATER_THAN_VALIDATOR = (field: FieldSpec): ValidatorSchema => {
  let minDate;
  const params = field.extra || {};
  if (params.min === 'now') {
    const today = new Date();
    minDate = new Date(today);
  } else {
    minDate = getDate(params.min).toDate();
  }
  const minPreviousDate = new Date(minDate);
  minPreviousDate.setDate(minDate.getDate() - 1);
  return {
    name: 'later-than',
    message: {
      key: 'form.error.date.later_than',
      context: {
        minDate: minPreviousDate.toLocaleDateString(),
      },
    },
    test: function (value: string) {
      if (!value) return true;
      const date = getDate(value);
      const isValid = +date.toDate() > +minDate;
      return isValid;
    },
  };
};

export const EARLIER_THAN_VALIDATOR = (field: FieldSpec): ValidatorSchema => {
  let maxDate;
  const params = field.extra || {};
  if (params.max === 'now') {
    const today = new Date();
    maxDate = new Date(today);
  } else {
    maxDate = getDate(params.max).toDate();
  }
  const maxNextDate = new Date(maxDate);
  maxNextDate.setDate(maxDate.getDate() + 1);
  return {
    name: 'earlier-than',
    message: {
      key: 'form.error.date.earlier_than',
      context: {
        maxDate: maxNextDate.toLocaleDateString(),
      },
    },
    test: function (value: string) {
      if (!value) return true;
      const date = getDate(value);
      const isValid = +date.toDate() < +maxDate;
      return isValid;
    },
  };
};

export const DATE_VALIDATOR = {
  name: 'date-validator',
  message: (v): string => {
    return checkPartialDate(v.value);
  },
  test: function (value: string) {
    if (!value) return true;
    const date = getDate(value);
    return date.isValid();
  },
};
