import { FieldSpec } from '@digigov/form/types';
import { ValidatorSchema } from '@digigov/form/validators/types';

const DEFAULT_IMAGE_MAX_HEIGHT_SIZE = 2000;
const DEFAULT_IMAGE_MAX_WIDTH_SIZE = 2000;
const DEFAULT_IMAGE_MIN_HEIGHT_SIZE = 100;
const DEFAULT_IMAGE_MIN_WIDTH_SIZE = 100;

function getImageDimensions(
  file: File
): Promise<{ width: number; height: number }> {
  return new Promise((resolve, reject) => {
    const reader = new FileReader();
    reader.onload = (event) => {
      const img = new Image();
      img.onload = () => {
        resolve({ width: img.width, height: img.height });
      };
      img.onerror = reject;
      const result = event?.target?.result;
      if (result && typeof result === 'string') {
        img.src = result;
      } else {
        reject(new Error('Invalid file format'));
      }
    };
    reader.onerror = reject;
    reader.readAsDataURL(file);
  });
}

export const IMAGE_DIMENSION_VALIDATOR = (
  field: FieldSpec
): ValidatorSchema => ({
  name: 'image-dimension-validator',
  message: (value: any) => {
    const minHeight =
      field?.extra?.limit?.minHeight || DEFAULT_IMAGE_MIN_HEIGHT_SIZE;
    const minWidth =
      field?.extra?.limit?.minWidth || DEFAULT_IMAGE_MIN_WIDTH_SIZE;
    const maxHeight =
      field?.extra?.limit?.maxHeight || DEFAULT_IMAGE_MAX_HEIGHT_SIZE;
    const maxWidth =
      field?.extra?.limit?.maxWidth || DEFAULT_IMAGE_MAX_WIDTH_SIZE;
    if (!value || value.length === 0) {
      return {
        name: 'image-validator',
        message: 'form.error.required',
      };
    } else {
      return {
        key: 'form.error.image.image_dimension_size',
        context: {
          maxHeight: maxHeight,
          maxWidth: maxWidth,
          minHeight: minHeight,
          minWidth: minWidth,
        },
      };
    }
  },
  test: async (value: File[]): Promise<boolean> => {
    if (!value || value.length === 0) {
      return true;
    }
    const file = value[0];
    const { width, height } = await getImageDimensions(file);
    const maxHeight =
      field?.extra?.limit?.maxHeight || DEFAULT_IMAGE_MAX_HEIGHT_SIZE;
    const maxWidth =
      field?.extra?.limit?.maxWidth || DEFAULT_IMAGE_MAX_WIDTH_SIZE;
    const minHeight =
      field?.extra?.limit?.minHeight || DEFAULT_IMAGE_MIN_HEIGHT_SIZE;
    const minWidth =
      field?.extra?.limit?.minWidth || DEFAULT_IMAGE_MIN_WIDTH_SIZE;
    if (!file) {
      return true;
    }
    try {
      if (height > maxHeight || width > maxWidth) {
        return false;
      }
      if (height < minHeight || width < minWidth) {
        return false;
      }
      return true;
    } catch (error) {
      console.error('Error getting image dimensions:', error);
      return false; // Return false in case of an error.
    }
  },
});
