import { FieldSpec } from '@digigov/form/types';
import { ValidatorSchema } from '@digigov/form/validators/types';

export type PhoneNumberType = 'landline' | 'mobile' | null;
// add more countries from here libphonenumber-js/metadata.full.json
const countryPhoneData = {
  GR: [
    '30',
    '00',
    '5005000\\d{3}|8\\d{9,11}|(?:[269]\\d|70)\\d{8}',
    [10, 11, 12],
    [
      ['(\\d{2})(\\d{4})(\\d{4})', '$1 $2 $3', ['21|7']],
      [
        '(\\d{4})(\\d{6})',
        '$1 $2',
        ['2(?:2|3[2-57-9]|4[2-469]|5[2-59]|6[2-9]|7[2-69]|8[2-49])|5'],
      ],
      ['(\\d{3})(\\d{3})(\\d{4})', '$1 $2 $3', ['[2689]']],
      ['(\\d{3})(\\d{3,4})(\\d{5})', '$1 $2 $3', ['8']],
    ],
    0,
    0,
    0,
    0,
    0,
    0,
    [
      [
        '2(?:1\\d\\d|2(?:2[1-46-9]|[36][1-8]|4[1-7]|5[1-4]|7[1-5]|[89][1-9])|3(?:1\\d|2[1-57]|[35][1-3]|4[13]|7[1-7]|8[124-6]|9[1-79])|4(?:1\\d|2[1-8]|3[1-4]|4[13-5]|6[1-578]|9[1-5])|5(?:1\\d|[29][1-4]|3[1-5]|4[124]|5[1-6])|6(?:1\\d|[269][1-6]|3[1245]|4[1-7]|5[13-9]|7[14]|8[1-5])|7(?:1\\d|2[1-5]|3[1-6]|4[1-7]|5[1-57]|6[135]|9[125-7])|8(?:1\\d|2[1-5]|[34][1-4]|9[1-57]))\\d{6}',
        [10],
      ],
      ['68[57-9]\\d{7}|(?:69|94)\\d{8}', [10]],
      ['800\\d{7,9}'],
      ['90[19]\\d{7}', [10]],
      ['70\\d{8}', [10]],
      0,
      ['5005000\\d{3}', [10]],
      0,
      0,
      ['8(?:0[16]|12|[27]5|50)\\d{7}', [10]],
    ],
  ],
};

function expandPhoneNumberStructure(compressed) {
  const countries = Object.keys(compressed);
  const expanded: any = {};
  for (const country of countries) {
    expanded[country] = {
      fixedLine: {
        possibleLengths: {
          _national: compressed[country][3].map(String),
        },
        nationalNumberPattern: compressed[country][11][0][0],
      },
      mobile: {
        nationalNumberPattern: compressed[country][11][1][0],
      },
      tollFree: {
        nationalNumberPattern: compressed[country][11][2][0],
      },
      premiumRate: {
        nationalNumberPattern: compressed[country][11][3][0],
      },
      sharedCost: {
        nationalNumberPattern: compressed[country][11][9][0],
      },
      personalNumber: {
        nationalNumberPattern: compressed[country][11][4][0],
      },
      uan: {
        nationalNumberPattern: compressed[country][11][6][0],
      },
      id: country,
      countryCode: compressed[country][0],
      internationalPrefix: compressed[country][1],
    };
  }
  return expanded;
}
const PHONENUMBER_SPEC = expandPhoneNumberStructure(countryPhoneData);
export function discoverPhoneType(phoneNumber: string, config: any): string {
  const cleanNumber = phoneNumber
    .replace(/\D/g, '')
    .replace(/\s/g, '')
    .replace(/^\+/, '')
    .replace(new RegExp('^' + config.internationalPrefix, ''), '')
    .replace(new RegExp('^' + config.countryCode, ''), '');
  const categories = {
    landline: config.fixedLine,
    mobile: config.mobile,
    'toll-free': config.tollFree,
    'premium-rate': config.premiumRate,
  };

  for (const [categoryName, categoryDetails] of Object.entries(categories)) {
    const pattern = new RegExp(
      '^(' + categoryDetails.nationalNumberPattern.replace(/\s/g, '') + ')$',
      ''
    );
    if (cleanNumber.match(pattern)) {
      return categoryName; // Returns the category name if the number matches the pattern
    }
  }

  // If no category matches, return 'Unknown'
  return 'unknown';
}
function getNumberType(phoneNumber: string, country: string) {
  const spec = PHONENUMBER_SPEC[country.toUpperCase()];
  if (!spec) {
    throw new Error(`Country ${country} is not supported`);
  }
  const type = discoverPhoneType(phoneNumber, spec);
  return type;
}

function isPhoneNumberValid(
  phoneNumber: string,
  countries: string[],
  types = ['mobile', 'landline']
) {
  return countries.some((country) => {
    const numberType = getNumberType(phoneNumber, country);
    if (numberType && types.includes(numberType)) {
      return true;
    }
    return false;
  });
}
function isNumberOfType(phoneNumber: string, country: string, type: string) {
  const numberType = getNumberType(phoneNumber, country);
  if (numberType === type) {
    return true;
  }
  return false;
}
export function validatePhoneNumber(
  phoneNumber: string,
  countries: string[] = ['gr'],
  typeOfPhoneNumber?: PhoneNumberType
): boolean {
  if (!countries || countries.length === 0) {
    return true;
  }
  return isPhoneNumberValid(
    phoneNumber,
    countries,
    typeOfPhoneNumber ? [typeOfPhoneNumber] : undefined
  );
}

function validateMobile(value): boolean {
  return isNumberOfType(value, 'gr', 'mobile');
}

export const MOBILE_PHONE_VALIDATOR = {
  name: 'mobile-phone-validator',
  message: 'form.error.mobile_phone',
  test: (value): boolean => {
    if (value) {
      return validateMobile(value);
    }
    return true;
  },
};

export const PHONE_NUMBER_VALIDATOR = (field: FieldSpec): ValidatorSchema => {
  const countryCodes = field?.extra?.countries;
  const typeOfPhone: PhoneNumberType | null = field?.extra?.phoneType;
  return {
    name: 'phone-number-validator',
    message: (): string => {
      if (typeOfPhone === 'mobile') {
        return 'form.error.mobile_phone';
      }
      if (typeOfPhone === 'landline') {
        return 'form.error.landline';
      } else {
        return 'form.error.phone_number';
      }
    },
    test: (value: string): boolean => {
      if (!value) {
        return true;
      }
      return validatePhoneNumber(value, countryCodes, typeOfPhone);
    },
  };
};
